﻿// Copyright (C) 2011 Oliver Sturm <oliver@oliversturm.com>. All rights reserved.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Linq.Expressions;
using System.Text;
using System.IO;

namespace QBE.Complete {
  public partial class _default : System.Web.UI.Page {
    protected void Page_Load(object sender, EventArgs e) {
      var predicate = ConstructCriteria(nameEdit.Text, areaEdit.Text, populationEdit.Text);
      IEnumerable<CountryInfo> data = new CountryInfoList( );
      if (predicate != null)
        data = data.Where(predicate);

      repeater.DataSource = data;
      repeater.DataBind( );
    }

    private Func<CountryInfo, bool> ConstructCriteria(string nameText, string areaText, string populationText) {
      var ciParam = Expression.Parameter(typeof(CountryInfo), "countryInfo");
      var expressions = new List<Expression> {
        ConstructNameCriteria(ciParam, nameText),
        ConstructAreaCriteria(ciParam, areaText),
        ConstructPopulationCriteria(ciParam, populationText)
      };
      var combined = Combine(expressions);

      // Purely for debugging purposes:
      if (combined != null) {
        var writer = new StringWriter( );
        ExpressionDumper.Output(combined, writer);
        filterLabel.Text = writer.ToString( );
      }
      else
        filterLabel.Text = "";

      // Depending on circumstances, it might make sense to return this expression
      // without compiling it first - then it could be fed into the LINQ translation
      // mechanisms and potentially executed against a server datasource.
      return combined != null ?
        Expression.Lambda<Func<CountryInfo, bool>>(combined, ciParam).Compile( ) :
        null;
    }

    private Expression Combine(List<Expression> expressions) {
      Expression result = null;
      foreach (var expression in expressions) {
        if (result == null)
          result = expression;
        else if (expression != null)
          result = Expression.AndAlso(result, expression);
      }
      return result;
    }

    private Expression ConstructNameCriteria(ParameterExpression ciParam, string nameText) {
      // Goal: construct an expression similar to this:
      // countryInfo => countryInfo.Name.StartsWith(nameText)

      if (!String.IsNullOrEmpty(nameText))
        return Expression.Call(
          Expression.Property(ciParam, "Name"),
          typeof(String).GetMethod("StartsWith", new Type[] { typeof(String) }),
          Expression.Constant(nameText));

      return null;
    }

    private Expression ConstructAreaCriteria(ParameterExpression ciParam, string areaText) {
      int area;
      if (Int32.TryParse(areaText, out area)) {
        // Goal: construct an expression similar to this:
        // countryInfo => countryInfo.AreaKM2 > area
        return Expression.GreaterThan(
          Expression.Property(ciParam, "AreaKM2"),
          Expression.Constant(area));
      }
      return null;
    }

    private Expression ConstructPopulationCriteria(ParameterExpression ciParam, string populationText) {
      int population;
      if (Int32.TryParse(populationText, out population)) {
        // Goal: construct an expression similar to this:
        // countryInfo => countryInfo.Population > population
        return Expression.GreaterThan(
          Expression.Property(ciParam, "Population"),
          Expression.Constant(population));
      }
      return null;
    }
  }
}